/**
***     CipherApp.c
***
***     ̃\[XR[h́AG[XgbvTXyhȂǂ̏Ă܂B
***     Copyright (C) 2006 SophiaCradle Incorporated.
***     All rights reserved.
**/

#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"
#include "AEEShell.h"
#include "Common.h"
#include "CipherApp.bid"
#include "AEESecurity.h"
#include "AEERSA.h"

typedef struct CipherApp {
    AEEApplet a; // Avbg\̂̐擪o͕K AEEApplet ^ɂ邱ƁB
    Common common; //`p

    IRSA* rsa; // IRSA C^[tF[X̃CX^Xϐ
    byte source[64];
    byte* encode;
    byte* decode;
    int size;
    uint32 result;
    AEECallback callback;
} CipherApp;

static boolean CipherApp_HandleEvent(CipherApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean OnAppStart(CipherApp* app);
static boolean OnAppStop(CipherApp* app);
static boolean OnKey(CipherApp* app, uint16 key);
static void ARC4Cipher(CipherApp *app);
static void ARC4MakeKeyHash(CipherApp* app, const char* key, byte* hashKey);
static void ARC4Encrypt(CipherApp *app, const byte* hashKey, const char* source, int size, byte* encode);
static void ARC4Decrypt(CipherApp *app, const byte* hashKey, const byte* encode, int size);
static void RSACipher(CipherApp* app);
static void RSAEncrypt(CipherApp* app);
static void RSADecrypt(CipherApp* app);

int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
    *ppObj = NULL;
    if (ClsId == AEECLSID_CIPHERAPP) {
        if (AEEApplet_New(sizeof(CipherApp), ClsId, pIShell, po, (IApplet**)ppObj, (AEEHANDLER)CipherApp_HandleEvent, NULL) == TRUE) {
            return AEE_SUCCESS;
        }
    }
    return EFAILED;
}

static boolean CipherApp_HandleEvent(CipherApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    switch (eCode) {
        case EVT_APP_START:
            return OnAppStart(app);

        case EVT_APP_STOP:
            return OnAppStop(app);

        case EVT_KEY:
            return OnKey(app, wParam);
    }
    return FALSE;
}

// AvbgJnƂɌĂяoB
static boolean OnAppStart(CipherApp* app)
{
    IShell* shell = app->a.m_pIShell;
    IDisplay* display = app->a.m_pIDisplay;

    app->rsa = NULL;
    STRCPY((char*)app->source, "This is RSA test message.");

    COMMON_Init(&app->common, display, shell, 32768);
    COMMON_WriteString(&app->common, "Push Select Key to test MD5 and ARC4\nPush 1 Key to test RSA\n");
    COMMON_Draw(&app->common);
    return TRUE;
}

// AvbgIƂɌĂяoB
static boolean OnAppStop(CipherApp* app)
{
    COMMON_Free(&app->common);
    return TRUE;
}

// L[ꂽƂɌĂяoB
static boolean OnKey(CipherApp* app, uint16 key)
{
    switch (key) {
        case AVK_SELECT:
            ARC4Cipher(app);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_1:
            RSACipher(app);
            return TRUE;

        case AVK_DOWN:
            COMMON_NextLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_UP:
            COMMON_PreviousLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;
    }
    return FALSE;
}

// MD5  ARC4 gÍ
static void ARC4Cipher(CipherApp *app)
{
    char key[] = "Key Cipher"; // ÍAŎgp錮
    byte hashKey[MD5_LENGTH + 1];
    char source[] = "This is MD5 HASH and ARC4 test message.";
    byte encode[sizeof(source)];
    int size = sizeof(source);

    COMMON_ClearBuffer(&app->common);
    COMMON_WriteString(&app->common, "Push Select Key to test MD5 and ARC4\nPush 1 Key to test RSA\n");

    ARC4MakeKeyHash(app, key, hashKey);
    ARC4Encrypt(app, hashKey, source, size, encode);
    ARC4Decrypt(app, hashKey, encode, size);
    return;
}

// MD5 gČnbV
static void ARC4MakeKeyHash(CipherApp* app, const char* key, byte* hashKey)
{
    IShell* shell = app->a.m_pIShell;
    IHash* hash;
    char buffer[256];
    int keySize = STRLEN(key);
    int hashSize = MD5_LENGTH;
    int i;
    int j;

    // IHash CX^X쐬
    hash = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_MD5, (void**)&hash);
    // Hash s
    IHASH_Update(hash, (byte*)key, keySize);
    // ʎ擾
    IHASH_GetResult(hash, hashKey, &hashSize);
    // ʕ\
    i = SPRINTF(buffer, "");
    j = 0;
    while (j < MD5_LENGTH) {
        i += SPRINTF(buffer + i, "%02x ", hashKey[j++]);
    }
    COMMON_WriteString(&app->common, "key source : \"");
    COMMON_WriteString(&app->common, key);
    COMMON_WriteString(&app->common, "\"\nkey : ");
    COMMON_WriteString(&app->common, buffer);
    COMMON_WriteString(&app->common, "\n");
    IHASH_Release(hash);
    return;
}

// ArcFour (RC4 ݊) gÍ
static void ARC4Encrypt(CipherApp *app, const byte* hashKey, const char* source, int size, byte* encode)
{
    IShell* shell = app->a.m_pIShell;
    ICipher* cipher;
    int hashSize = MD5_LENGTH;
    char buffer[256];
    int i;
    int j;

    // ICipher C^[tFCX쐬
    cipher = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_ARC4, (void**)&cipher);
    // ÍŎgp錮ݒ
    ICIPHER_Init(cipher, hashKey, hashSize);
    // ÍÕf[^o
    COMMON_WriteString(&app->common, "source : \"");
    COMMON_WriteString(&app->common, source);
    COMMON_WriteString(&app->common, "\"\n");

    // Í
    ICIPHER_Cipher(cipher, (byte*)source, size, encode, &size);
    // Íf[^o
    i = SPRINTF(buffer, "");
    j = 0;
    while (j < size) {
        i += SPRINTF(buffer + i, "%02x ", encode[j++]);
    }
    COMMON_WriteString(&app->common, "encode : ");
    COMMON_WriteString(&app->common, buffer);
    COMMON_WriteString(&app->common, "\n");

    // ICipher C^[tFCX
    ICIPHER_Release(cipher);
    return;
}

// ArcFour (RC4 ݊) g
static void ARC4Decrypt(CipherApp *app, const byte* hashKey, const byte* encode, int size)
{
    IShell* shell = app->a.m_pIShell;
    ICipher* cipher;
    int hashSize = MD5_LENGTH;
    byte* decode = (byte*)MALLOC(size);

    // ICipher C^[tFCX쐬
    cipher = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_ARC4, (void**)&cipher);
    // Ŏgp錮ݒ
    ICIPHER_Init(cipher, hashKey, hashSize);
    // 
    ICIPHER_Cipher(cipher, encode, size, decode, &size);
    // f[^o
    COMMON_WriteString(&app->common, "decode : ");
    COMMON_WriteString(&app->common, (char*)decode);
    COMMON_WriteString(&app->common, "\n");

    // obt@
    FREE(decode);
    // ICipher C^[tFCX
    ICIPHER_Release(cipher);
    return;
}

// RSA gÍ 
static void RSACipher(CipherApp* app)
{
    const unsigned char mod512[] = {
        0xA9, 0x59, 0x31, 0x2E, 0xBA, 0x86, 0xDB, 0x0B,
        0x2B, 0x98, 0xAE, 0x0A, 0x15, 0x9F, 0xF6, 0x3E,
        0x01, 0x77, 0x0D, 0xBC, 0xBB, 0xDA, 0xEA, 0x47,
        0xFB, 0x7E, 0xF5, 0xB6, 0xB1, 0xA1, 0xE9, 0xC9,
        0x57, 0x9D, 0x2A, 0xBD, 0xA8, 0x46, 0x1C, 0x44,
        0x89, 0x3F, 0x52, 0xBD, 0x13, 0x84, 0xE4, 0x37,
        0x7C, 0xDD, 0x67, 0x2E, 0xD9, 0xC5, 0x0B, 0xC2,
        0xC5, 0x63, 0x6E, 0xC9, 0xEE, 0x5F, 0xE2, 0xBB
    }; // W
    unsigned char publicE = 0x03; // J
    IShell* shell = app->a.m_pIShell;

    if (app->rsa == NULL) {
        COMMON_ClearBuffer(&app->common);
        COMMON_WriteString(&app->common, "Push Select Key to test MD5 and ARC4\nPush 1 Key to test RSA\n");
        COMMON_WriteString(&app->common, "source : \"");
        COMMON_WriteString(&app->common, (char*)app->source);
        COMMON_WriteString(&app->common, "\n");

        CALLBACK_Init(&app->callback, RSAEncrypt, app);
        // IRSA CX^X쐬
        app->rsa = NULL;
        ISHELL_CreateInstance(shell, AEECLSID_RSA, (void**)&app->rsa);
        // 
        IRSA_Init(app->rsa, mod512, 64, &publicE, sizeof(publicE));
        // Í
        IRSA_ModExp(app->rsa, app->source, 64, &app->encode, &app->size, &app->result, &app->callback);
        // ʂ̎擾ƕ\̓R[obN҂B
    }
    return;
}

// RSA gÍȈ
static void RSAEncrypt(CipherApp* app)
{
    const unsigned char mod512[] = {
        0xA9, 0x59, 0x31, 0x2E, 0xBA, 0x86, 0xDB, 0x0B,
        0x2B, 0x98, 0xAE, 0x0A, 0x15, 0x9F, 0xF6, 0x3E,
        0x01, 0x77, 0x0D, 0xBC, 0xBB, 0xDA, 0xEA, 0x47,
        0xFB, 0x7E, 0xF5, 0xB6, 0xB1, 0xA1, 0xE9, 0xC9,
        0x57, 0x9D, 0x2A, 0xBD, 0xA8, 0x46, 0x1C, 0x44,
        0x89, 0x3F, 0x52, 0xBD, 0x13, 0x84, 0xE4, 0x37,
        0x7C, 0xDD, 0x67, 0x2E, 0xD9, 0xC5, 0x0B, 0xC2,
        0xC5, 0x63, 0x6E, 0xC9, 0xEE, 0x5F, 0xE2, 0xBB
    }; // W
    unsigned char exp512[] = {
        0x70, 0xE6, 0x20, 0xC9, 0xD1, 0xAF, 0x3C, 0xB2,
        0x1D, 0x10, 0x74, 0x06, 0xB9, 0x15, 0x4E, 0xD4,
        0x00, 0xFA, 0x09, 0x28, 0x7D, 0x3C, 0x9C, 0x2F,
        0xFC, 0xFF, 0x4E, 0x79, 0xCB, 0xC1, 0x46, 0x85,
        0x24, 0x2E, 0x9C, 0x9E, 0xBD, 0xBA, 0x97, 0xB0,
        0x37, 0xC7, 0x26, 0x6C, 0x41, 0xDB, 0x5B, 0x16,
        0xD7, 0xB1, 0x76, 0x03, 0xF0, 0xEA, 0xAF, 0x23,
        0xFC, 0x19, 0x82, 0x8E, 0x1E, 0x3A, 0xFA, 0x9B
    }; // 閧
    char buffer[256];
    int i;
    int j;

    // ʕ\
    i = SPRINTF(buffer, "");
    j = 0;
    while (j < app->size) {
        SPRINTF(buffer + i, "%02x ", app->encode[j++]);
        i += 3;
    }
    COMMON_WriteString(&app->common, "encode : ");
    COMMON_WriteString(&app->common, buffer);
    COMMON_WriteString(&app->common, "\n");
    COMMON_Draw(&app->common);

    CALLBACK_Init(&app->callback, RSADecrypt, app);
    // ď
    IRSA_Init(app->rsa, mod512, 64, exp512, 64);
    // 
    IRSA_ModExp(app->rsa, app->encode, app->size, &app->decode, &app->size, &app->result, &app->callback);
    // ʂ̎擾ƕ\̓R[obN҂B
    return;
}

// RSA gȈ
static void RSADecrypt(CipherApp* app)
{
    // ʕ\
    COMMON_WriteString(&app->common, "decode : \"");
    COMMON_WriteString(&app->common, (char*)app->decode);
    COMMON_WriteString(&app->common, "\"\n");
    // IRSA 
    IRSA_Release(app->rsa);
    app->rsa = NULL;
    COMMON_Draw(&app->common);
    return;
}
